/*----------------------------------------------------------------------------
                                INFORMATION    
  ----------------------------------------------------------------------------*/

/* Waist belt creator Version 2
   Written by Kakiemon @ Thingiverse
   Rolf Jethon
   Licensed under the GPL v3 Terms and conditions.

   HOW TO USE THIS FILE :  I recommend to use the OpenScad Customizer

   -- Solution 1 :
   Open this file with OpenScad software (free and open-source)
   http://www.openscad.org/downloads.html
   Choose your parameters in the SETTINGS chapter (see below)
   Press F5 to compile and see the resulting clip
   Check your hook's global size in the console window (below viewing area)
   If OK, press F6 to render
   Go to : File => Export => Export as STL : choose filename and location
   Save the STL file of your customized belt
   Send this STL file to your 3D printer

   -- Solution 2 :
   This file had been optimized for the Customizer whether Thingiverse or OpenSCAD
   OpenScad: Just change the parameters in the customizer section
   Thingiverse:
   You will need to have a Thingiverse Account, then :
   Go to object's page on Thingiverse
   Click on the button "Open in Customizer"
   Choose your settings in parameters tabs
   Save the STL file of your customized part
   Process and print this STL file for your 3D printer
*/
/*------------------------------------------------------------------------------
With this program you can create waist belts of meander shaped form of various 
sizes and shapes. Not too much attention  needs to be taken to parametrize the
parts. In most cases the resulting parts are still sensful if the obvious size
relations remain reasonable. 
==================
 IMPORTANT NOTES:
==================
Do not use any brim or raft for printing, otherwise you have a lot of cleaning
work to do. When creating the belt, it is important to use a marerial, that
has a reasonable high inter layer and bed adhesion such as PETG. Also it is 
important that the resulting GCODE of your slicer is creating a continuous
movement. 
If you do not respect the previous notes, probaply your belt will not be 
stable enough to really use it. Usually the belt thickness must be a 
multiple of your nozzle width, means the belt needs to consist of 1 to 4 
continuous layers without fast positioning movements within the windings.
After printing the belt needs to be glued onto the buckle base. I recommend
cyan acrylate glue (super glue) for that.
Rendering and slicing of the belt may take quite some time because of the 
complexity. On my Ryzen7 CPU the reference settings resut in a rendering time
of approx. 20 minutes!

Improvements in Version 2
--------------------------
- Fixes some small calculation errors.
- Fixed the pressure curveof the buckle cover so it can be shifted in its 
position over the buckle base. In Version 1 the Buckle only worked wihout a 
stop radius. Means mounting with a stop radius was impossible.
- Added a cut view to check the shape of the part fits

------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------
                                SETTINGS          
 ----------------------------------------------------------------------------*/

/* [General] */
// Which parts to show 0=all parts 1=cone 2=ring 3=cap 4=all parts print friendly
visible=4;//[0:All mounted,1:Belt only,2:Buckle base only,3:Buckle cover only,4:Print all,5:Cut view]
// The length of the belt ends in mm when in all mounted mode
showlength=80;
// The gap to give parts the required play
playgap=.25;//[0:.01:1]
/*[Belt Properties Sizes in mm]*/
// The belt is wrapped around this diameter for printing in mm
printdiam=150;//[100:300]
// The length of the belt in mm
beltlength=1100;
// The height of the belt
beltheight=35;//[1:1:100]
// The diameter of the inner meander curve
meanderdiam=1;//[.1:.1:8]
// The thickness of the meander wall
beltthick=.75;//[.1:.05:4]
// Distance in between two meander curves
bowgap=.5;//[.1:.1:8]
// Distance in between the first and the second belt print winding turn
turngap=.7;//[0.1:.1:3]
// accuracy segments
$fn=20;
/*[Buckle Parameters]*/
// The number of curves that grip into the belt
gripcurves=3;//[3:6]
// Thickness of the buckle cover wall in mm
bucklecoverwall=2;//[1.5:.1:6]
// The rounding radius of the buckle cover in mm
roundingradius=2;//[0:.1:6]
// The buckel wall thickness
bucklebasewall=1.5;//[1:.1:4]
// Depth of the stop edge reaching into the buckle base
bucklestopdepht=1.5;//[0:.1:4]
// Buckle guide width beside the stop notch in % of the buckle base width
slideguidewidth=20;//[10:.5:40]
// Minimum length of the buckle in mm
minbucklelength=30;//[20:1:80]
/*[Hidden]*/
arra=[0];
arrb=[0];

// calculate some base sizes
gripcv=gripcurves-1;
outermeander=meanderdiam+2*beltthick; // outer meander diameter
cutheight=beltheight+.2; // a little larger to cut off the core
neutralradius=(meanderdiam+beltthick)/2; // the neutral radius of the meander curve
effgap=(bowgap < meanderdiam) ? bowgap : meanderdiam-.01; // minimize parameter mistakes
halfdist=[meanderdiam/2+beltthick+effgap/2,sqrt((meanderdiam+beltthick)*(meanderdiam+beltthick)-(meanderdiam/2+beltthick+effgap/2)*(meanderdiam/2+beltthick+effgap/2))]; // Half distance in between two meander curves
meanderthick=halfdist[1]+(2*neutralradius)+beltthick; // Thickness of the belt
dot_dist=2*halfdist[0]; // Distance in between two meander curves
dot_dist2=dot_dist*dot_dist; // dot_dist²
dots=floor(beltlength/dot_dist)+1; // Number of meandercurves
growthperturn=(meanderthick+turngap); // Growth for one belt turn the next to get a spiral
growth=growthperturn/((printdiam*PI)/dot_dist); // Growth for one meander curve to the next
cutangle=asin((meanderdiam/2+beltthick+effgap/2)/(meanderdiam+beltthick)); // Angle to cut a meander tube to get the segment
cutdeform=tan(cutangle); // scale factor to get the cutoff body
bucklebaseheight=beltheight+2*bucklebasewall+2*bucklestopdepht+2*playgap;
bucklebasewidth=bucklebasewall+2*meanderthick+2*playgap;
bucklelength=gripcv*dot_dist+6 *dot_dist;
effbucklelength=bucklelength<=minbucklelength ? minbucklelength:bucklelength;
notchwidth_2=(bucklebasewidth-bucklebasewidth/100*slideguidewidth)*(bucklebasewidth-bucklebasewidth/100*slideguidewidth);
stopradius=bucklestopdepht/2+notchwidth_2/(8*bucklestopdepht);
cutblockhalf=(effbucklelength-(gripcv+1.5)*dot_dist+effgap/2)/2;
r1=outermeander/2+playgap;
b=(r1*2>=dot_dist) ? (meanderthick-(r1-sqrt(r1*r1-(dot_dist*dot_dist)/4)))/2:(meanderthick/2+halfdist[1]/2)/2;
ra=b/2+cutblockhalf*2*cutblockhalf*2/(8*b);

if (visible==0){ // show all mounted
  allmounted();
}    
if(visible==1){ // show only the belt
  belt();
}
if(visible==2){ // show only the buckle base
  translate([0,0,effbucklelength])  
  rotate([-90,0,0])  
  bucklebase();    
}
if(visible==3){ // show only the buckle cover
  rotate([90,0,0])  
  bucklecover();    
}
if(visible==4){ // show all for printing
  translate([-(bucklebasewidth+(bucklecoverwall+playgap)+turngap),bucklebaseheight/2,0])  
  rotate([90,0,0])  
  bucklecover();    
  translate([turngap,-bucklebaseheight/2,effbucklelength])  
  rotate([-90,0,0])  
  bucklebase();
  translate([0,0,beltheight/2])  
  belt();  
}    
if (visible==5){ // show mounted parts in a cut view
 sl=2*showlength+2*effbucklelength;     
 difference(){  
 allmounted();
 translate([-sl/2,-showlength,15])   
 cube(sl); 
 translate([-sl/2,-showlength,-sl+14.9])   
 cube(sl);            
 }
}

//The following two self calling functions are necessary, because OpenScad
//does not allow to modify variable values within the program.

function fill(arra,i=0) = // calculate the winding of each S and put into array
    i >= dots ? arra : (
        fill( 
            concat(arra,(arra[i-1]+acos(((printdiam/2+i*growth)*(printdiam/2+i*growth)+(printdiam/2+(i-1)*growth)*(printdiam/2+(i-1)*growth)-dot_dist2)/(2*(printdiam/2+i*growth)*(printdiam/2+(i-1)*growth))))),
        i+1)
    );
    
function fill2(arrb,j=0) = // calculate the rotation of each S and put it into array 
    j >= dots ? arrb : (
        fill2( 
            concat(arrb,(acos((dot_dist2+(printdiam/2+(j-1)*growth)*(printdiam/2+(j-1)*growth)-(printdiam/2+j*growth)*(printdiam/2+j*growth))/(2*(dot_dist)*(printdiam/2+(j-1)*growth))))),
        j+1)
    );
    
module allmounted(){ //show all parts mounted 
  translate([bucklebasewidth-meanderthick/2,dot_dist*gripcurves-dot_dist/4,bucklebaseheight/2])   
  rotate([0,0,180])   
  showbelt(showlength);  
  translate([meanderthick/2,dot_dist*gripcurves-dot_dist/4+dot_dist/2,bucklebaseheight/2])   
  rotate([0,0,180])   
  showbelt(showlength/2);     
  translate([2*b+meanderthick/2,effbucklelength-dot_dist,bucklebaseheight/2])   
  showbelt(showlength); 
  bucklebase();
  bucklecover();   
}

module bucklecover(){
//bucklebase();    
union(){    
difference(){    
translate([roundingradius-bucklecoverwall-playgap,roundingradius,roundingradius-bucklecoverwall-playgap])    
minkowski()
{   
  cube([bucklebasewidth+(bucklecoverwall+playgap)*2-2*roundingradius,effbucklelength-roundingradius*2,bucklebaseheight+(bucklecoverwall+playgap)*2-2*roundingradius]);
  sphere(r=roundingradius,$fn=60);
}
translate([-playgap,-.01,-playgap])
cube([bucklebasewidth+2*playgap,effbucklelength+.02,bucklebaseheight+2*playgap]);
}
  intersection(){
  translate([-playgap-.01,-.01,-playgap-.01])
  cube([bucklebasewidth+2*playgap+.02,bucklebasewall+.02,bucklebaseheight+2*playgap+.02]);
  translate([bucklebasewidth/2,0,-stopradius+bucklestopdepht])
  rotate([-90,0,0])
  cylinder(h=bucklebasewall,r=stopradius-playgap,$fn=80); // guidenotch lower
  }
  intersection(){
  translate([-playgap-.01,-.01,-playgap-.01])
  cube([bucklebasewidth+2*playgap+.02,bucklebasewall+.02,bucklebaseheight+2*playgap+.02]);
  translate([bucklebasewidth/2,0,bucklebaseheight+stopradius-bucklestopdepht])
  rotate([-90,0,0])
  cylinder(h=bucklebasewall,r=stopradius-playgap,$fn=80); //guidenotch upper    
  }   
  intersection(){    // convex belt guide curve part
    translate([-playgap-.01,effbucklelength-cutblockhalf,(bucklebaseheight-beltheight+playgap*2)/2])
    cube([bucklebasewidth-bucklebasewall-meanderthick-2*playgap,cutblockhalf+.01,beltheight-2*playgap]);
    translate([-ra+b*2-playgap,effbucklelength])    
    cylinder(h=bucklebaseheight,r=ra,$fn=280);    
  }    
  difference(){      // concave belt guide curve part
    translate([-playgap-.01,effbucklelength-cutblockhalf*2,(bucklebaseheight-beltheight+playgap*2)/2])
    cube([bucklebasewidth-bucklebasewall-meanderthick-2*playgap,cutblockhalf+.01,beltheight-2*playgap]);
    translate([ra-playgap,effbucklelength-2*cutblockhalf])    
    cylinder(h=bucklebaseheight,r=ra,$fn=280);  
  }     
 } 
}    

module bucklebase(){
  difference(){
    cube([bucklebasewidth,effbucklelength,bucklebaseheight]);
    translate([bucklebasewidth-(meanderthick+playgap)+.01,-.01,0])
    gripcurve(gripcv);
    translate([meanderthick+playgap-.01,-.01,bucklebaseheight])
    rotate([0,180,0])    
    gripcurve(gripcv+1);        
    translate([bucklebasewidth/2,effbucklelength-bucklebasewall-.01,-stopradius+bucklestopdepht])
    rotate([90,0,0])
    cylinder(h=effbucklelength-bucklebasewall,r=stopradius,$fn=80); // guidenotch lower
    translate([bucklebasewidth/2,effbucklelength-bucklebasewall-.01,bucklebaseheight+stopradius-bucklestopdepht])
    rotate([90,0,0])
    cylinder(h=effbucklelength-bucklebasewall,r=stopradius,$fn=80); //guidenotch upper  
    intersection(){    // concave belt guide curve part
      translate([-.01,effbucklelength-cutblockhalf,(bucklebaseheight-beltheight-playgap*2)/2])
      cube([bucklebasewidth-bucklebasewall,cutblockhalf+.01,beltheight+2*playgap]);
      translate([-ra+meanderthick+2*b+playgap,effbucklelength])    
      cylinder(h=bucklebaseheight,r=ra,$fn=280);    
    }    
    difference(){      // convex belt guide curve part
      translate([-.01,effbucklelength-cutblockhalf*2,(bucklebaseheight-beltheight-playgap*2)/2])
      cube([bucklebasewidth-bucklebasewall,cutblockhalf+.01,beltheight+2*playgap]);         
      translate([ra+meanderthick+playgap,effbucklelength-2*cutblockhalf])    
      cylinder(h=bucklebaseheight,r=ra,$fn=280);  
    }    
  }  
}    
module gripcurve(gripcv){ // creates the grip curves for the belt inside the bucke base (subtracted from the buckle base)   
    for(i=[0:1:gripcv]){
      translate([outermeander/2+playgap,i*dot_dist+outermeander/2,(bucklebaseheight-beltheight-playgap*2)/2])  
      cylinder(h=beltheight+2*playgap,d=outermeander+2*playgap,$fn=80);
    } 
    translate([outermeander/2+playgap,0,(bucklebaseheight-beltheight-playgap*2)/2])
    cube([meanderthick-outermeander/2+playgap,dot_dist*gripcv+outermeander+playgap,beltheight+2*playgap]);   
}    

module showbelt(showlength){ // display the straight show belt for all mounted
  for(i=[0:1:floor(showlength/dot_dist+1)]){
    translate([0,i*dot_dist+effgap/2,0])
      full(); 
  }
}  
module belt(){ // display the winded belt
angles=fill(arra,1);
angles2=fill2(arrb,1);

for(i=[0:1:dots-2]){ // place the meander segments in a spiral form
  union(){  
  rotate([0,0,angles[i]])    
  translate([printdiam/2+i*growth,0,0])  
  rotate([0,0,90-angles2[i+1]])   
full();}   
}
}
module full(){ // put two half segments together to get one full meander segment
  rotate([0,0,cutangle])
  translate([neutralradius,0,0])   
  rotate([0,0,90-cutangle])
union(){    
half();       // lower bow
translate([halfdist[0],halfdist[1],0]) // Mirror and shift one bow so it fits
rotate([0,0,180])
half();  // upper bow
}
}

module half(){ // make a half meander segment
difference(){
cylinder(h=beltheight,d=outermeander,center=true);  //outerdiameter of meander bow  
cylinder(h=cutheight,d=meanderdiam,center=true); // inner diameter of meander bow
scale([cutdeform,1,1])    // cutoff cube defomed to achieve a well fit of the cut bows
rotate([0,0,45])    
translate([0,0,-cutheight/2]) // center Z axis   
cube([outermeander,outermeander,cutheight]);
}
rotate([0,0,cutangle])
translate([0,neutralradius,0])
cylinder(h=beltheight,d=beltthick,center=true);
rotate([0,0,-cutangle])
translate([0,neutralradius,0])
cylinder(h=beltheight,d=beltthick,center=true);
}
